// -------------------------------------------------------
//    enjxp_SimPassion for FlightSimParts.eu
//    wendy - FlightSimparts.eu
// -------------------------------------------------------
// 
//  Wendy:      Updated : Sunday December 8th, 2019
//              Created a new sketch for Serial.Read mode only
//  enjxp_SimPassion 	: Updated Wednesday November 20th, 2019
//							Removed Loop limit for each module
//							Added switching between Serial.read mode and SI_Message_Port mode
//							Disabled values backup inside the sketch
//  Wendy 				: Updated Saturday November 16, 2019 by Wendy FlightSimparts.eu for use with or without Airmanager
//							Improved and clarified coding by using C++ struct
//							Added digits letters handling like A N S E W ...
//							Improved digits number handling
//							Improved decimal point handling
//							Added to allow either Serial.read or SI_Message_Port way, inside the same coding
//  enjxp_SimPassion 	: Updated Saturday October 26th, 2019
//  enjxp_SimPassion 	: Updated Wednesday October 23rd, 2019
//  enjxp_SimPassion 	: Updated Sunday September 8th, 2019
//  enjxp_SimPassion 	: Initial script : Tuesday March 19th, 2018

//  This sketch handle the Specific MicroDriver from FlightSimParts.eu, which allow 1 to 8 Digits of 7-Segment displays
//  http://www.flightsimparts.eu/sim_microdriver.htm
//  http://www.flightsimparts.eu/shop_microdriver7segment.htm


// ARDUINO code to display Simulators values using an Arduino Board and MicroDisplay drivers with 2 embedded 74HC595D and 1 ULN2803A chipsets
// the sketch can handle currently 6 display sets of up to 8 digts
// the sketch can be enhanced to handle more MicroDriver displays

//  INITS           // To be updated manually, accordingly to specific usage
//-------------------------
//  CONSTANTS           // To be updated manually, accordingly to Digits number connected to FlightSimPart.eu Micro Driver

struct DriverConnect
{
   int Clock;
   int Latch;
   int Data;
   byte Digits;
   byte Decimal;
};

struct Digits  //now for 6 modules, to add a extra module, add "String G;" ...
{
  String A;
  String B;
  String C;
  String D;
  String E;
  String F;
};

// MODULE A : clockPin/latchPin/dataPin/Digits/DecimalPosition values : To be updated manually, accordingly to personal wiring

const DriverConnect ModuleA = {31, 32, 33, 5, 3}; // green Pin connected to SH_CP of 74HC595, blue Pin connected to ST_CP of 74HC595, purple Pin connected to DS of 74HC595
const DriverConnect ModuleB = {28, 29, 30, 5, 3}; // green, blue, purple
const DriverConnect ModuleC = {25, 26, 27, 6, 3}; // green, blue, purple
const DriverConnect ModuleD = {22, 23, 24, 6, 3}; // green, blue, purple
const DriverConnect ModuleE = {40, 41, 42, 8, 0}; // green, blue, purple
const DriverConnect ModuleF = {43, 44, 45, 6, 0}; // green, blue, purple

Digits ToDisplay 				= {"01234", "01234", "012345", "012345", "01234567", "012345"}; 
//                  				on Mod A, on Mod B, on Mod C, on Mod D, on Mod E, on Mod F 

int max_dig_nbr       			= 8;

//do not change
int digitpos[8]       			= {1, 2, 4, 8, 16, 32, 64, 128};

//-------------------------
//  Variables List

String command;

String digits_A       			= "";    // 10800 - 11795  // NAV1 STANDBY
String digits_B       			= "";    // 10800 - 11795  // NAV1 ACTIVE
String digits_C       			= "";    // 118000 - 136975  // COM1 STANDBY
String digits_D       			= "";    // 118000 - 136975  // COM1 ACTIVE
String digits_E       			= "";    // ELEC PANEL Lower Digits
String digits_F       			= "";    // ELEC PANEL Upper Digits

String mem_digits_A   			= "";
String mem_digits_B   			= "";
String mem_digits_C   			= "";
String mem_digits_D   			= "";
String mem_digits_E   			= "";
String mem_digits_F   			= "";

int dig_start         			= 1;
int dig_value         			= 0;
byte SIMP_rcv_byte_value     	= 0;
String SIMP_rcv_string_value 	= "";
int SIMP_rcv_int_value       	= 0;
float SIMP_rcv_float_value   	= 0.0;
int num_ind                  	= 0;
uint16_t message_ident       	= 0;
char num_char;
char num_charA;
String identRead = "";
char iRead;

const int dig_val_nbr = 17; //  Added for modularity in case of any enhancement = Digits Values Number
int number[dig_val_nbr] = {252,96,218,242,102,182,190,224,254,246,238,236,182,158,124,2,0}; // base 10 equivalents for digits 0~9 and 'A' / 'N' / 'S' / 'E' / 'W' / '-' / ' ' additional chars --> FlightSimParts.eu specific Micro Driver
  
  /* Relying on the FlightSimParts.eu specific MicroDrivers matrix
  
  For units only

  0 = 252
  1 = 96
  2 = 218
  3 = 242
  4 = 102
  5 = 182
  6 = 190
  7 = 224
  8 = 254
  9 = 246

  For units plus decimal point, take the previous values and add +1
  Automatically set according to DriverConnect values

  0. = 253
  1. = 97
  2. = 219
  3. = 243
  4. = 103
  5. = 183
  6. = 191
  7. = 225
  8. = 255
  9. = 247

  Symbols
  
  "A" = 238
  "N" = 236
  "S" = 182
  "E" = 158
  "W" = 124 	--> "U"
  "-" = 2
  " " = 0 		--> Blank (Space char)

  */


void setup() {

	Serial.begin(9600);

  //set pins to output so you can control the shift register
	pinMode(ModuleA.Clock,  OUTPUT);
	pinMode(ModuleA.Latch,  OUTPUT);
	pinMode(ModuleA.Data,	OUTPUT);
	pinMode(ModuleB.Clock,  OUTPUT);
	pinMode(ModuleB.Latch,  OUTPUT);
	pinMode(ModuleB.Data,	OUTPUT);
	pinMode(ModuleC.Clock,  OUTPUT);
	pinMode(ModuleC.Latch,  OUTPUT);
	pinMode(ModuleC.Data,	OUTPUT);
	pinMode(ModuleD.Clock,  OUTPUT);
	pinMode(ModuleD.Latch,  OUTPUT);
	pinMode(ModuleD.Data,	OUTPUT);
	pinMode(ModuleE.Clock,  OUTPUT);
	pinMode(ModuleE.Latch,  OUTPUT);
	pinMode(ModuleE.Data,	OUTPUT);
	pinMode(ModuleF.Clock,  OUTPUT);
	pinMode(ModuleF.Latch,  OUTPUT);
	pinMode(ModuleF.Data,	OUTPUT);

}

//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
//    Display Values on 7-Segment Digits
//-----------------------------------------------------------------------------
  /*  For Reference : mandatory similar four lines to allow display of values in this particular order :
  *   
    digitalWrite(latchPin, LOW);
    shiftOut(dataPin, clockPin, MSBFIRST, value);           // First statement for Value to be displayed
    shiftOut(dataPin, clockPin, MSBFIRST, digitposition);   // Second statement for Digit position on which to display value
    digitalWrite(latchPin, HIGH);
  */
//-----------------------------------------------------------------------------

void loop() {

		if(Serial.available()){
			command = Serial.readStringUntil('\n');
			ToDisplay.A = command;
 
		}

//-----------------------------------------------------------------------------
//				MAIN LOOP PART
//-----------------------------------------------------------------------------
    
    //Display loops
    for (int num=0; num < max_dig_nbr; num++)
    {
      // Module A
      if (mem_digits_A != ToDisplay.A)
      {
        digitalWrite(ModuleA.Data, 0);
        digitalWrite(ModuleA.Clock, 0);
        mem_digits_A = ToDisplay.A;
      }
        num_char = ToDisplay.A[num]; 
        if (isDigit(num_char)) 
        {          
          num_ind = (String(num_char)).toInt(); 
          dig_value = number[num_ind]; 
          if (num == ModuleA.Decimal-1) {dig_value = dig_value + 1;}; //set decimal according to DriverConnect settings
        } else
        {
          if (num_char == 'A') {dig_value = number[10];};
          if (num_char == 'N') {dig_value = number[11];};
          if (num_char == 'S') {dig_value = number[12];};
          if (num_char == 'E') {dig_value = number[13];};
          if (num_char == 'W') {dig_value = number[14];};
          if (num_char == '-') {dig_value = number[15];};
          if (num_char == ' ') {dig_value = number[16];};
        }
        digitalWrite(ModuleA.Latch, LOW);
        shiftOut(ModuleA.Data, ModuleA.Clock, MSBFIRST, dig_value); // value
        shiftOut(ModuleA.Data, ModuleA.Clock, MSBFIRST, digitpos[num]);
        digitalWrite(ModuleA.Latch, HIGH);

     //Module B
      if (mem_digits_B != ToDisplay.B)
      {
        digitalWrite(ModuleB.Data, 0);
        digitalWrite(ModuleB.Clock, 0);
        mem_digits_B = ToDisplay.B;
      }
        num_char = ToDisplay.B[num]; 
        if (isDigit(num_char)) 
        {          
          num_ind = (String(num_char)).toInt(); 
          dig_value = number[num_ind]; 
          if (num == ModuleB.Decimal-1) {dig_value = dig_value + 1;}; //set decimal according to DriverConnect settings
        } else
        {
          if (num_char == 'A') {dig_value = number[10];};
          if (num_char == 'N') {dig_value = number[11];};
          if (num_char == 'S') {dig_value = number[12];};
          if (num_char == 'E') {dig_value = number[13];};
          if (num_char == 'W') {dig_value = number[14];};
          if (num_char == '-') {dig_value = number[15];};
          if (num_char == ' ') {dig_value = number[16];};
        }
        digitalWrite(ModuleB.Latch, LOW);
        shiftOut(ModuleB.Data, ModuleB.Clock, MSBFIRST, dig_value); // value
        shiftOut(ModuleB.Data, ModuleB.Clock, MSBFIRST, digitpos[num]);
        digitalWrite(ModuleB.Latch, HIGH);
   
      //Module C
      if (mem_digits_C != ToDisplay.C)
      {
        digitalWrite(ModuleC.Data, 0);
        digitalWrite(ModuleC.Clock, 0);
        mem_digits_C = ToDisplay.C;
      }
        num_char = ToDisplay.C[num]; 
        if (isDigit(num_char)) 
        {          
          num_ind = (String(num_char)).toInt(); 
          dig_value = number[num_ind]; 
          if (num == ModuleC.Decimal-1) {dig_value = dig_value + 1;}; //set decimal according to DriverConnect settings
        } else
        {
          if (num_char == 'A') {dig_value = number[10];};
          if (num_char == 'N') {dig_value = number[11];};
          if (num_char == 'S') {dig_value = number[12];};
          if (num_char == 'E') {dig_value = number[13];};
          if (num_char == 'W') {dig_value = number[14];};
          if (num_char == '-') {dig_value = number[15];};
          if (num_char == ' ') {dig_value = number[16];};
        }
        digitalWrite(ModuleC.Latch, LOW);
        shiftOut(ModuleC.Data, ModuleC.Clock, MSBFIRST, dig_value); // value
        shiftOut(ModuleC.Data, ModuleC.Clock, MSBFIRST, digitpos[num]);
        digitalWrite(ModuleC.Latch, HIGH);

      //Module D
      if (mem_digits_D != ToDisplay.D)
      {
        digitalWrite(ModuleD.Data, 0);
        digitalWrite(ModuleD.Clock, 0);
        mem_digits_D = ToDisplay.D;
      }
        num_char = ToDisplay.D[num]; 
        if (isDigit(num_char)) 
        {          
          num_ind = (String(num_char)).toInt(); 
          dig_value = number[num_ind]; 
          if (num == ModuleD.Decimal-1) {dig_value = dig_value + 1;}; //set decimal according to DriverConnect settings
        } else
        {
          if (num_char == 'A') {dig_value = number[10];};
          if (num_char == 'N') {dig_value = number[11];};
          if (num_char == 'S') {dig_value = number[12];};
          if (num_char == 'E') {dig_value = number[13];};
          if (num_char == 'W') {dig_value = number[14];};
          if (num_char == '-') {dig_value = number[15];};
          if (num_char == ' ') {dig_value = number[16];};
        }
        digitalWrite(ModuleD.Latch, LOW);
        shiftOut(ModuleD.Data, ModuleD.Clock, MSBFIRST, dig_value); // value
        shiftOut(ModuleD.Data, ModuleD.Clock, MSBFIRST, digitpos[num]);
        digitalWrite(ModuleD.Latch, HIGH);

      //Module E
      if (mem_digits_E != ToDisplay.E)
      {
        digitalWrite(ModuleE.Data, 0);
        digitalWrite(ModuleB.Clock, 0);
        mem_digits_E = ToDisplay.E;
      }
        num_char = ToDisplay.E[num]; 
        if (isDigit(num_char)) 
        {          
          num_ind = (String(num_char)).toInt(); 
          dig_value = number[num_ind]; 
          if (num == ModuleE.Decimal-1) {dig_value = dig_value + 1;}; //set decimal according to DriverConnect settings
        } else
        {
          if (num_char == 'A') {dig_value = number[10];};
          if (num_char == 'N') {dig_value = number[11];};
          if (num_char == 'S') {dig_value = number[12];};
          if (num_char == 'E') {dig_value = number[13];};
          if (num_char == 'W') {dig_value = number[14];};
          if (num_char == '-') {dig_value = number[15];};
          if (num_char == ' ') {dig_value = number[16];};
        }
        digitalWrite(ModuleE.Latch, LOW);
        shiftOut(ModuleE.Data, ModuleE.Clock, MSBFIRST, dig_value); // value
        shiftOut(ModuleE.Data, ModuleE.Clock, MSBFIRST, digitpos[num]);
        digitalWrite(ModuleE.Latch, HIGH);

      //Module F
      if (mem_digits_F != ToDisplay.F)
      {
        digitalWrite(ModuleF.Data, 0);
        digitalWrite(ModuleF.Clock, 0);
        mem_digits_F = ToDisplay.F;
      }
        num_char = ToDisplay.F[num]; 
        if (isDigit(num_char)) 
        {          
          num_ind = (String(num_char)).toInt(); 
          dig_value = number[num_ind]; 
          if (num == ModuleF.Decimal-1) {dig_value = dig_value + 1;}; //set decimal according to DriverConnect settings
        } else
        {
          if (num_char == 'A') {dig_value = number[10];};
          if (num_char == 'N') {dig_value = number[11];};
          if (num_char == 'S') {dig_value = number[12];};
          if (num_char == 'E') {dig_value = number[13];};
          if (num_char == 'W') {dig_value = number[14];};
          if (num_char == '-') {dig_value = number[15];};
          if (num_char == ' ') {dig_value = number[16];};
        }
        digitalWrite(ModuleF.Latch, LOW);
        shiftOut(ModuleF.Data, ModuleF.Clock, MSBFIRST, dig_value); // value
        shiftOut(ModuleF.Data, ModuleF.Clock, MSBFIRST, digitpos[num]);
        digitalWrite(ModuleF.Latch, HIGH);
    }
}
